library("rstan")
options(mc.cores = parallel::detectCores())
rstan_options(auto_write = TRUE)
Sys.setenv(LOCAL_CPPFLAGS = '-march=native')

library("rethinking")
-----------------------------------------------
  #EXAMPLE - GRID APPROXIMATION
  
  #allows us to estimate the posterior with a finite value of paramter values
  #at any particular point value of a paramter, p', multiply it by the likelihood of p' - repeat for each value in the grid - generates an approximation of the posterior
  
  
  #define grid: step 1 - you are deciding how many points to use in estiamting the posterior and then making a list of the paramter values on the grid
  p_grid = seq(from=0, to=1, length.out=1000)

#define prior: step 2  ----------------------> here the prior is a flat prior 
prior= rep(1,1000)

#compute likelihood at each value in grid: step 3
likelihood = dbinom(6, size = 9, prob = p_grid)


#compute product of likelihood and prior: step 4
unstd.posterior = likelihood*prior


#standardize the posterioir, so it sums to 1: step 5
posterior = unstd.posterior/sum(unstd.posterior)
---------------------------------------------------------------
  plot(posterior)

---------------------------------------------------------------
  
  samples = sample(p_grid, prob = posterior,size = 1e4, replace = TRUE)    #drawing 10,000 samples from the posterior - samples is pulling values from the vector p_grid with each value having the proabbility of the posterioir above 
-------------------------------------------------------------
  plot(samples)
--------------------------------------------
  library(rethinking)
dens(samples)

-----------------------------------------
  sum(posterior[p_grid<.5])        #add up posterioir probability where p <.5

--------------------------------------
  sum(samples>.5 & samples<.75)/1e4     #____% of the probability lies between .5 and .75
------------------------------------
  quantile(samples, .8)                #asking where in samples does the 80th percentile lie? 
-------------------------------
  quantile(samples, c(.05,.95))
-----------------------------------------
  HPDI(samples, prob = .5)      #highest posterior density interval - 50% - narrowest region with 50% of the posterioir probability. this will always include the most probable parameter value
------------------------------------
  
  #ESTIMATES 
  ------------------------------------
  p_grid[which.max((posterior))]    #compute a maximum a posterior estiamte (MAP)
------------------------------------  
  chainmode(samples, adj=.01)    #if you have samples from the posterior, approximates the same point as MAP
------------------------------------
  mean(samples)                  # average for samples object
------------------------------------
  median(samples)               #median for samples object
------------------------------------
  
  
  
---------------------------------------
#loading the data
library(rethinking)
data(Howell1)             #access the data set howell1
d=Howell1                 #make howell1 a data frame called d
d2= d[d$age >=18,]


d2$weight=scale(d2$weight)


d2$height=scale(d2$height)



#fit a  model of height as a linear function of weight
m1=map(
  alist(
    height~dnorm(mu, sigma),
    mu<-a + b*weight,
    a~dnorm(0,1),
    b~dnorm(0,1),
    sigma~dunif(0,5)
  ),
  data=d2)


precis(m1) #model coefficients 

plot(precis(m1)) #plot model coefficients 

plot(height~weight, d2, col=col.alpha(rangi2, .5)) #plot the data
abline(m1) #draw a line using the model

########################################
library(brms)

fit1 <- brm(formula = 
              height ~ weight,
            data = d2, family = gaussian(),
            prior = c(prior(normal(0, 1), class = Intercept),
                      prior(normal(0, 1), class = b),
                      prior(uniform(0, 5), class = sigma)),
            iter = 2000, chains = 4)


posterior_summary(fit1)

plot(fit1)

plot(conditional_effects(fit1), points = TRUE)